/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#ifdef         __SCRIPT_ENABLE
#include <sys/msg.h>
#endif
#ifdef         __WINDOWS_ENABLE    
#include <windows.h>   
#endif  
#include "Commun.h"
#include "Debug.h"
#include "Initialisation_Com.h"
#include "Reseau.h"
#include "Communication.h"
#include "Log.h"
#include "Mem.h"
#include "Socket.h"

extern struct Struct_Modele_Communication Liste_Communication[];
	

gint Calculer_Ckecksum(gint *Donnee, gint Taille_Donnee){
	gint i,Ckecksum;
	
	Ckecksum = 0;
	for (i=0;i<(Taille_Donnee/sizeof(gint));i++)
		Ckecksum = Donnee[i] + Ckecksum;
		
	return Ckecksum;
}

void Log_Envoie_Transmission(struct Struct_Liste_Reseau *Reseau,gchar *Texte_Log){
	if (Reseau!=NULL)
		Log(Reseau->Log_Send,Texte_Log);
	MYFREE(Texte_Log);
}

void Communication_Envoie_Donnee (gint Pid_Agent, gint ID_Info, gpointer * Donnee, gint Taille_Donnee, struct Struct_Liste_Reseau *Reseau)
{
	struct struct_Info_TCP Info_TCP;
	int NbOctet,Socket;

#ifdef         __SCRIPT_ENABLE
	struct struct_Info Info;

	if ((ID_Info==RETOUR_SCRIPT) || (ID_Info==RETOUR_SCRIPT_TERMINER)){
		if ((Taille_Donnee + sizeof (gint)) >= TAILLE_MAX_DONNEE_BRUT)
			DEBUG_ERROR("Taille du message >=%d",TAILLE_MAX_DONNEE_BRUT);
		Info.type = Pid_Agent;	
		Info.ID_Info = ID_Info;
		memcpy (Info.Donnee, Donnee, Taille_Donnee);
		if (msgsnd (File_Info, (void *) &Info, (Taille_Donnee + sizeof (gint)), 0) < 0)
			printf ("msgsnd : %s\n", strerror (errno));
		return;
	}
#endif


#ifdef __DEBUG_COM
	Log_Envoie_Transmission(Reseau,g_strdup_printf("[%s] %d\n[%s] Ckecksum %08X\n",Liste_Communication[ID_Info].Debug,Taille_Donnee,Liste_Communication[ID_Info].Debug,Info_TCP.Ckecksum));
#endif    

	if (Reseau != NULL) {
		Socket = Reseau->Socket_Agent;
		Reseau->Octet_Envoyer_Agent = Reseau->Octet_Envoyer_Agent + Taille_Donnee;
		if  (Reseau->Etat_Connection != ETAT_CONNECTED){
			printf("--- Socket disconnected\n");	
			return;
		}		
	}		
	else 
		Socket = Socket_Agent_Init; 

	if ((Taille_Donnee + sizeof (gint)) >= TAILLE_MAX_DONNEE_BRUT)
		DEBUG_ERROR("Taille du message >=%d",TAILLE_MAX_DONNEE_BRUT);

	Info_TCP.ID_Info = ID_Info;
	Info_TCP.Taille_Donnee = Taille_Donnee;	
	Info_TCP.Ckecksum = Calculer_Ckecksum((gint *)Donnee,Taille_Donnee);
	memcpy (Info_TCP.Donnee, Donnee, Taille_Donnee);

	if (Waiting_Write_Socket(Socket, TIMEOUT_SOCKET_AGENT) != SOCK_OK){
		if (Reseau != NULL) {
			DEBUG_WARNING("send (Resource temporarily unavailable)");
			Change_Etat_Connection_Reseau(Reseau,ETAT_DISCONNECTED);
			return;
		}
		else
			DEBUG_ERROR("send (Resource temporarily unavailable)"); 
	}
	SEND(NbOctet,Socket, &Info_TCP, sizeof (struct struct_Info_Entete)+Taille_Donnee);
	if (NbOctet < 0){
#ifdef __DEBUG_COM
		Log_Envoie_Transmission(Reseau,g_strdup_printf("[%s] %d\n[%s] Ckecksum %08X   #### send NbOctet < 0  (%d)[%s]\n",Liste_Communication[ID_Info].Debug,Taille_Donnee,Liste_Communication[ID_Info].Debug,Info_TCP.Ckecksum,errno, strerror (errno)));
#endif
		if (Reseau != NULL) {
			DEBUG_WARNING("Ckecksum %08X   #### send NbOctet < 0  (%d)[%s]\n",Liste_Communication[ID_Info].Debug,Taille_Donnee,Liste_Communication[ID_Info].Debug,Info_TCP.Ckecksum,errno, strerror (errno));
			Change_Etat_Connection_Reseau(Reseau,ETAT_DISCONNECTED);
			return;
		}
		else
			DEBUG_ERROR("Ckecksum %08X   #### send NbOctet < 0  (%d)[%s]\n",Liste_Communication[ID_Info].Debug,Taille_Donnee,Liste_Communication[ID_Info].Debug,Info_TCP.Ckecksum,errno, strerror (errno));
	}
	if (NbOctet != (sizeof (struct struct_Info_Entete)+Taille_Donnee)){
#ifdef __DEBUG_COM
		Log_Envoie_Transmission(Reseau,g_strdup_printf("send NbOctet != (sizeof (struct struct_Info_Entete)+Taille_Donnee) (%d)[%s]", errno, strerror (errno));
#endif
		if (Reseau != NULL) {
			DEBUG_WARNING("send NbOctet != (sizeof (struct struct_Info_Entete)+Taille_Donnee) (%d)[%s]", errno, strerror (errno));
			Change_Etat_Connection_Reseau(Reseau,ETAT_DISCONNECTED);
			return;
		}
		else
			DEBUG_ERROR("send NbOctet != (sizeof (struct struct_Info_Entete)+Taille_Donnee) (%d)[%s]", errno, strerror (errno));
	}
}
