/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <gtk/gtk.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include "Execute.h"
#include "Initialisation_Com.h"
#include "Configuration.h"
#include "Debug.h"
#include "main.h"
#include "Mem.h"
#ifdef         __WINDOWS_ENABLE
#include <windows.h>
#else
#include <sys/stat.h>
#endif 

#ifndef         __WINDOWS_ENABLE    
int Test_Droit_Execution(const char *executable){
static struct stat _stat;

	if(stat(executable,&_stat)) return FALSE;
	if((_stat.st_mode&(S_IXGRP|S_IXUSR|S_IXOTH|S_ISUID|S_ISGID)) 
	 	&&S_ISREG(_stat.st_mode)) return TRUE;
	return FALSE;
}

char *Cherche_Chemin_Executable(char *commande)
{
	char *path = NULL,*chemin_commande = NULL,*s = NULL,*saveptr = NULL;

	path = getenv("PATH");

	if(!path) return NULL; 
	path = g_strdup(path);
	if(!path) return NULL;
	s = strtok_r(path,":", &saveptr);// on recupere la première repertoire dans le path
	while(s) {
		chemin_commande = g_strdup_printf ("%s/%s",s,commande);
	
		if(Test_Droit_Execution(chemin_commande)) { 
			MYFREE(path);
		
			return chemin_commande;
		}
		MYFREE (chemin_commande);
		s = strtok_r(0,":", &saveptr);//cherche dans le prochain chemin dans path...
	}
	MYFREE (path);

	return NULL;
}

#define LISTE_NAVIGATEUR "firefox,mozilla,epiphany,konqueror,iceweasel,opera,dillo,browser" 
gchar *Recherche_Navigateur(){
gchar *liste = NULL,*s = NULL,*navigateur = NULL,*saveptr = NULL;
	liste = g_strdup(LISTE_NAVIGATEUR);
	s = strtok_r(liste,",", &saveptr);// on recupere la premier navigateur
	while(s) {
	
		if ((navigateur = Cherche_Chemin_Executable(s))!=NULL){
			MYFREE (liste);
		
			return navigateur;
		}
		s = strtok_r(0,",", &saveptr);
	}
	MYFREE (liste);
	return NULL;
}

void Affiche_Page_Html(gchar * url){
gchar *navigateur = NULL;
	
	if ((navigateur = Recherche_Navigateur())!=NULL){
		Execute_Commande(navigateur,url);
		MYFREE (navigateur);
	}
}


void Execute_Commande(gchar * command, gchar * param){
int       pid_fils;	

	pid_fils = fork ();	
	if (pid_fils == 0) {   
	
		execlp (command, command, param, NULL);
		_exit (0);
	}
}
#endif  


#ifdef         __SCRIPT_ENABLE
#ifndef         __WINDOWS_ENABLE    
void Execute_Programme_Externe (gchar * path, gchar * param, struct Struct_Equipement *Equipement)
{
	int       pid_fils;

	ERROR_if (path == NULL);
	ERROR_if (param == NULL);
	ERROR_if (Equipement == NULL);

	if (!g_file_test(path,G_FILE_TEST_EXISTS)){
		debug_printf (D_INFO_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Can't find the '%s' program!", path);
		return;
	}
	pid_fils = fork ();
	if (pid_fils == 0) {
#define VARIABLE_IP "_IP_"
#define VARIABLE_MAC "_MAC_"
		gchar     Command[255], NCommand[255], *pVariable;

	
		strcpy (Command, path);
		strcat (Command, " ");
		strcat (Command, param);
		if ((pVariable = strstr (Command, VARIABLE_IP)) != 0) {
		
			strncpy (NCommand, Command, pVariable - Command);
			NCommand[pVariable - Command] = '\0';
		
			strcat (NCommand, Equipement->Adresse_Ip_Char);
		
			strcat (NCommand, pVariable + strlen (VARIABLE_IP));
		
			strcpy (Command, NCommand);
		}
		if ((pVariable = strstr (Command, VARIABLE_MAC)) != 0) {
			strncpy (NCommand, Command, pVariable - Command);
			NCommand[pVariable - Command] = '\0';
			strcat (NCommand, Equipement->Adresse_MAC_Char);
			strcat (NCommand, pVariable + strlen (VARIABLE_MAC));
			strcpy (Command, NCommand);
		}
	
		system (Command);
		_exit (0);
	}
}
#endif  
#endif




void Lance_Nouveau_Agent_Local () {
#ifndef         __WINDOWS_ENABLE
	
	if (Config_Path_Daemon != NULL){
		pid_t     pid_fils; 
		gchar     Pid_Parent[6];
	
		sprintf (Pid_Parent, "%d", Pid_AutoScan);
	
		debug_printf (D_INFO_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Start : %s\n",Config_Path_Daemon);
		pid_fils = fork ();
		if (pid_fils == 0) {
			_exit (0);
		}
	}
#endif
	
#ifdef         __WINDOWS_ENABLE   	
	HWND Handle; 
	int err; 


	if ((err=ShellExecute (NULL, "open", Config_Path_Daemon, NULL, NULL, SW_SHOWNORMAL))<=32){ 

		switch (err){ 
		case 0 : 
			DEBUG_ERROR("System was out of memory, executable file was corrupt, or relocations were invalid.(%d)",err); 
			break; 
		case 2 : 
            DEBUG_ERROR("File was not found.(%d)",err); 
			break; 
   		case 3 : 
			DEBUG_ERROR("Path was not found.(%d)",err); 
			break; 
   		case 5 : 
            DEBUG_ERROR("Attempt was made to dynamically link to a task, or there was a sharing or network-protection error.(%d)",err); 
			break; 
		case 6 : 
			DEBUG_ERROR("Library required separate data segments for each task.(%d)",err); 
			break; 
		case 8 : 
			DEBUG_ERROR("There was insufficient memory to start the application.(%d)",err); 
			break; 
		case 10 : 
            DEBUG_ERROR("Windows version was incorrect.(%d)",err); 
			break; 
		case 11 : 
	        DEBUG_ERROR(" Executable file was invalid. Either it was not a Windows application, or there was an error in the .exe image.(%d)",err); 
			break; 
		case 12 : 
			DEBUG_ERROR("Application was designed for a different operating system.(%d)",err); 
			break; 
		case 13 : 
			DEBUG_ERROR("Application was designed for MS-DOS 4.0.(%d)",err); 
			break; 
		case 14 : 
			DEBUG_ERROR("Type of executable file was unknown.(%d)",err); 
			break; 
		case 15 : 
            DEBUG_ERROR("Attempt was made to load a real-mode application (developed for an earlier version of Windows).(%d)",err); 
			break; 
		case 16 : 
            DEBUG_ERROR("Attempt was made to load a second instance of an  executable file containing multiple data segments that were not marked read-only.(%d)",err); 
			break; 
		case 19 : 
            DEBUG_ERROR("Attempt was made to load a compressed executable file. The file must be decompressed before it can be loaded.(%d)",err); 
			break; 
		case 20 : 
            DEBUG_ERROR("Dynamic-link library (DLL) file was invalid. One of the DLLs required to run this application was corrupt.(%d)",err); 
			break; 
		case 21 : 
            DEBUG_ERROR("Application requires Microsoft Windows 32-bit extensions.(%d)",err); 
			break; 
		default : 
		     DEBUG_ERROR("?? (%d)",err); 
			}
	} 

#endif   
}
