/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Socket.h"
#include "Semaphore.h"
#include "Debug.h"
#include "Win32.h"

gint Waiting_Write_Socket (int Socket, int Nb_Seconde)
{
	fd_set fd_write;
	socklen_t lon;
    struct timeval tv;
	int retval = 0,valopt = 0,Temps_Passe = 0;

	if (Show_Info_Log)
		DEBUG_INFO( "Waiting Socket [Write] %d  : %d seconde(s)",Socket,Nb_Seconde);
	if (Socket<=0){
		DEBUG_WARNING( "The descriptor is not a socket [%d]",Socket);
		return SOCK_ERROR;
	}
	if (Nb_Seconde<=0)
		DEBUG_WARNING("Nb Seconde invalide (%d)",Nb_Seconde);
	
	while (Temps_Passe<Nb_Seconde){
		FD_ZERO(&fd_write);
		FD_SET(Socket, &fd_write);
   		tv.tv_sec = 1;
    	tv.tv_usec = 0;
 		retval = select(Socket + 1,NULL, &fd_write, NULL, &tv);
    	if (retval==1){
    		lon = sizeof(int);
    		if (getsockopt(Socket, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon) < 0) {
				DEBUG_WARNING( "Error in getsockopt() %d - %s\n", errno, strerror(errno));
        		return SOCK_ERROR;
			}
			if (valopt) {
				DEBUG_WARNING( "Error in delayed connection() %d - %s",valopt, strerror(valopt));
				return SOCK_ERROR;
			}
			if (Show_Info_Log)
				DEBUG_INFO( "Ecriture Possible sur la socket (%d)",Socket);
			return SOCK_OK;
		}
    	else {
			DEBUG_WARNING("Impossible écrure depuis 1 secondes  retval=%d sur %d",retval,Socket);
			sleep(1);
		}
		Temps_Passe++;
	}
	return SOCK_ERROR;
}



int Waiting_Read_Socket (int Socket, int Nb_Seconde)
{
	fd_set fd_read;
	socklen_t lon;
    struct timeval tv;
	int retval = 0,valopt = 0, Temps_Passe = 0;

	if (Show_Info_Log)
		DEBUG_INFO( "Waiting Socket [Read] %d  : %d seconde(s)",Socket,Nb_Seconde);
	if (Socket<=0){
		DEBUG_WARNING( "The descriptor is not a socket");
		return SOCK_ERROR;
	}
	if (Nb_Seconde<=0){
		DEBUG_WARNING("Nb Seconde invalide");
		return SOCK_ERROR;
	}
	while (Temps_Passe<Nb_Seconde){
		FD_ZERO(&fd_read);
		FD_SET(Socket, &fd_read);
   		tv.tv_sec = 1;				
    	tv.tv_usec = 0;
 		retval = select(Socket + 1,&fd_read, NULL, NULL, &tv);
    	if (retval==1){
#ifndef         __WINDOWS_ENABLE
			lon = sizeof(int); 
    		if (getsockopt(Socket, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon) < 0) {
				DEBUG_WARNING("Error in getsockopt() %d - %s\n", errno, strerror(errno));
        		return SOCK_ERROR;
			}
			if (valopt) { 
				switch (valopt){
					case 104 : 
										DEBUG_INFO( "Connection reinitialiser par l'hote mais des données sont peut etre presente (%d)  %d - %s",Socket ,valopt, strerror(valopt));
										return SOCK_RESET_BY_PEER;
										break;
					case 32 :    
										DEBUG_WARNING("tcpwrapped ??? %d - %s",valopt, strerror(valopt));
										return SOCK_BROKEN_PIPE;
					default:
										DEBUG_WARNING("Error in delayed connection() %d - %s",valopt, strerror(valopt));
										return -valopt;
					}
			}
#endif
			if (Show_Info_Log)
				DEBUG_INFO("Donnee presente sur la socket (%d)",Socket);
			return SOCK_OK;
		}
    	else {
			DEBUG_WARNING("Aucune donnée depuis 1 secondes  retval=%d sur %d",retval,Socket);
			sleep(1);
		}
		Temps_Passe++;
	}
	return SOCK_ERROR; 
}


int recvfrom_control (int Socket, char *buffer, size_t length, int flags, struct sockaddr_in *address, int address_len){
	int Recu=0;

	if (Socket<=0){
		DEBUG_WARNING("The descriptor is not a socket");
		return -1;
	}
	if ((address == NULL) && (address_len != 0)){
		DEBUG_WARNING( "sockaddr non initialise");
		return -1;
	}
	if ((Recu = recvfrom (Socket, buffer, length, flags, (struct sockaddr *) address, (socklen_t *)&address_len)) < 0) {
		DEBUG_WARNING("Socket recvfrom  : %s",strerror (errno));
		return -1;
	}
	Nouveau_Octet_Recu (Recu);
	return Recu;
}

int sendto_control (int Socket, const void *Packet, size_t len, int flags, const struct sockaddr *to, socklen_t tolen){
	int	 retour = 0;

	Nouveau_Octet_Envoye (len);
	if (Socket<=0){
		DEBUG_WARNING( "The descriptor is not a socket");
		return -1;
	}
	if ((to == NULL) && (tolen != 0)){
		DEBUG_WARNING( "sockaddr non initialise");
		return -1;
	}
	if ((retour = sendto (Socket, Packet, len, flags, to, tolen)) < 0) {
		DEBUG_WARNING( "Socket sendto  : %s   tolen(%d)", strerror (errno),tolen);
		return -1;
	}
	if (retour != len) {
		DEBUG_WARNING( "Number of characters sent !=    (%s)", strerror (errno));
		return -1;
	}
	return retour;
}


int connect_control (int Socket,struct sockaddr_in *sock,unsigned char *Ip, int Port) {

	if (Socket<=0){
		DEBUG_WARNING( "The descriptor is not a socket");
		return SOCK_ERROR;
	}
	if (sock == NULL){
		DEBUG_WARNING("sock invalide");
		return SOCK_ERROR;
	}
	if (connect (Socket, (struct sockaddr *) sock, sizeof (struct sockaddr_in)) < 0) {
#ifndef         __WINDOWS_ENABLE
		if (errno != EINPROGRESS) {
			DEBUG_INFO("%s : Equipement : [%s] port:[%d] ", strerror (errno), Ip, Port);
			return errno;
		}
		DEBUG_INFO("Socket avec etat en progression : [%s] port:[%d]  socket (%d)",Ip, Port,Socket);
#else
		if (WSAGetLastError() != WSAEWOULDBLOCK) {
			DEBUG_INFO( "connect() [%s] port:[%d]  (%d) failed with error %d   %d",Ip, Port,Socket, WSAGetLastError(),SOCKET_ERROR);
			return WSAGetLastError();
		}
		DEBUG_INFO("Socket avec etat en progression : [%s] port:[%d]  (%d)",Ip, Port,Socket);
#endif
	}
	return SOCK_OK;
}


unsigned int Get_Nb_Socket_Ouvert(){
unsigned int Nb_Socket = 0;

	PROTECT_CODE (Semaphore_Control_Usage_Socket);
	Nb_Socket = Nb_Socket_Ouverte;
	DEPROTECT_CODE (Semaphore_Control_Usage_Socket);
	return Nb_Socket;
}

int socket_control (int domain, int type, int protocol)
{
	int   Socket = 0;

	if ((Socket = socket (domain, type, protocol)) == -1) {
		DEBUG_WARNING("Socket : Create Failed (%s)\n",strerror(errno));
		return -1;
	}
	PROTECT_CODE (Semaphore_Control_Usage_Socket);
	Nb_Socket_Ouverte++;
	DEPROTECT_CODE(Semaphore_Control_Usage_Socket);
	return Socket;
}

int close_control (int Socket)
{
	int       close_s = 0;

	if (Socket<=0){
		DEBUG_WARNING( "The descriptor is not a socket");
		return -1;
	}
#ifndef         __WINDOWS_ENABLE
	if ((close_s = close (Socket)) == -1)
		DEBUG_ERROR ("Socket : Close Failed (%s)\n",strerror(errno));
#else

	        if ((close_s=closesocket(Socket))==-1){
	   		int error = WSAGetLastError();   
			switch (error){
				case WSANOTINITIALISED:
					DEBUG_ERROR ("A successful WSAStartup call must occur before using this function.");
				break;
			case WSAENETDOWN:
					DEBUG_ERROR ("The network subsystem has failed.");
				break;
			case WSAENOTSOCK:
					DEBUG_ERROR ("The descriptor is not a socket.");
					break;
			case WSAEINPROGRESS:
					DEBUG_ERROR ("A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function.");
					break;
			case WSAEINTR:
					DEBUG_ERROR ("The (blocking) Windows Socket 1.1 call was canceled through WSACancelBlockingCall.");
					break;
			case WSAEWOULDBLOCK:
					DEBUG_ERROR ("The socket is marked as nonblocking, but the l_onoff member of the linger structure is set to non-zero and the l_linger member of the linger structure is set to a nonzero timeout value.");
					break;
			default:
					DEBUG_ERROR ("Erreur Inconnu = %d",error);
			}
		}
#endif
	PROTECT_CODE (Semaphore_Control_Usage_Socket);
	Nb_Socket_Ouverte--;
	DEPROTECT_CODE (Semaphore_Control_Usage_Socket);
	return close_s;
}

int shutdown_socket (int Socket)
{
	if (Socket<=0){
		DEBUG_WARNING( "The descriptor is not a socket");
		return -1;
	}
	if (shutdown (Socket, SHUT_RDWR) == -1)
		DEBUG_WARNING ("Socket : Close Failed (%s)\n",strerror(errno));
	printf("shutdown_socket : %d\n",Socket);
}


int bind_control (int Socket,struct sockaddr * sock) {
	int       bind_s = 0;

	if ((bind_s =bind (Socket, sock, sizeof (struct sockaddr_in))) == -1)
		DEBUG_WARNING("Bind : %s", strerror (errno));
	return bind_s;
}

int listen_control (int Socket,int Nb_Max_Client) {
	int       listen_s = 0;

	if ((listen_s = listen (Socket, Nb_Max_Client)) == -1)
		DEBUG_WARNING("Listen : %s", strerror (errno));
	return listen_s;
}

#define F_SETSIG	10
void signal_socket(int Socket){
	int pid = getpid();
	/*if (ioctl(Socket, SIOCSPGRP, (char *) &pid) < 0)
		DEBUG_WARNING("ioctl: SIOCSPGRP");
	else
		DEBUG_INFO("signal_socket : OK");*/
	/*kill (pid, SIGUSR2);
	if (fcntl (Socket, F_SETSIG,SIGUSR2) == -1)
			DEBUG_ERROR ("%s", strerror (errno));*/
}

#ifndef         __WINDOWS_ENABLE
void set_NonBlock (int Socket,int block) {
	int       flags = 0;

	ERROR_if (Socket<=0);
	if (block == SOCKET_NON_BLOQUANT){
		if ((flags = fcntl (Socket, F_GETFL)) == -1)
			DEBUG_ERROR ("%s", strerror (errno));
		if (fcntl (Socket, F_SETFL, flags | O_NONBLOCK) == -1)
			DEBUG_ERROR ("%s", strerror (errno));
	}
	else
		DEBUG_ERROR("Non implémenté \n");
}
#else
void set_NonBlock (int Socket,int block) {
	ERROR_if (Socket<=0);
	if (block == SOCKET_NON_BLOQUANT){
		if (ioctlsocket(Socket, FIONBIO, (unsigned long *)&block))
			DEBUG_ERROR ("network: cannot block socket");
	}
	else
		DEBUG_ERROR ("Non implémenté \n");
}
#endif
