/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2010 Thierry Lagarde <thierry@autoscan-network.com>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#ifndef         __SOCKET
#define         __SOCKET

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include "config.h"

#ifdef         __SEMAPHORE_ENABLE
	#include <semaphore.h>
#endif

#ifndef         __WINDOWS_ENABLE
	#include <sys/socket.h>
	#include <arpa/inet.h>
#else
	#ifdef    __MSVC_ENABLE
		#define _WINSOCKAPI_
	#else
		#include <windows.h>
		#include <winsock2.h>
		#include <ws2tcpip.h>
	#endif
#endif

#ifdef         __PCAP_ENABLE
#include <pcap.h>
#endif

#ifdef __WINDOWS_ENABLE
#define SHUT_RDWR 2
#endif

unsigned int Nb_Socket_Ouverte;
int Show_Info_Log;

#define SOCK_BROKEN_PIPE 			-32
#define SOCK_ERROR						-1
#define SOCK_OK								1
#define SOCK_RESET_BY_PEER			-104
#define SOCKET_NON_BLOQUANT 	1
#define SOCKET_BLOQUANT 			0

int		Waiting_Write_Socket (int Socket, int Nb_Seconde);
int 		Waiting_Read_Socket (int Socket, int Nb_Seconde);
int 		sendto_control (int Socket, const void *Packet_Arp, size_t len, int flags, const struct sockaddr *to, socklen_t tolen);
int   	socket_control (int domain, int type, int protocol);
int   	close_control (int fd);
unsigned int Get_Nb_Socket_Ouvert();
void 	set_NonBlock (int sock,int block);
int bind_control (int Socket,struct sockaddr * sock) ;
int listen_control (int Socket,int Nb_Max_Client) ;
int connect_control (int Socket,struct sockaddr_in *sock,unsigned char *Ip, int Port);
void signal_socket(int Socket);

#define WAITING_WRITE_SOCKET(_Resultat,_Socket,_Nb_Seconde) G_STMT_START{							\
	_Resultat = Waiting_Write_Socket (_Socket,_Nb_Seconde);																\
	}G_STMT_END


#define WAITING_READ_SOCKET(_Resultat,_Socket,_Nb_Seconde) G_STMT_START{							\
	_Resultat = Waiting_Read_Socket (_Socket,_Nb_Seconde);																\
	}G_STMT_END


#ifndef         __WINDOWS_ENABLE
#define RECVFROM(_recu,_Socket,_buffer,_length,_flags,_address,_address_len) G_STMT_START{		\
	_recu = recvfrom_control (_Socket, _buffer, _length, _flags, _address, _address_len);					\
	if (_recu < 0)																																\
		DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));							\
	}G_STMT_END
#else
#define RECVFROM(_recu,_Socket,_buffer,_length,_flags,_address,_address_len) G_STMT_START{		\
	_recu = recvfrom_control (_Socket, _buffer, _length, _flags, _address, _address_len);					\
	if (_recu < 0)																																\
		DEBUG_ERROR("Erreur(%d)  %s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)",WSAGetLastError(), strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));							\
	}G_STMT_END
#endif	
		
		
#define CREATE_SOCKET(_Socket,_Ip,_port,_sock,_domain,_type,_protocol) G_STMT_START{					\
	if ((_Socket = socket_control (_domain, _type, _protocol)) < 0)														\
			DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement)); 	\
	memset ((char *) &_sock, 0, sizeof (struct sockaddr_in));															\
	_sock.sin_family = AF_INET;																											\
	_sock.sin_port = htons (_port);																										\
	if (_Ip != NULL)																															\
		_sock.sin_addr.s_addr = inet_addr((char *)_Ip); 																		\
	else																																				\
		_sock.sin_addr.s_addr = htonl (INADDR_ANY);																			\
	}G_STMT_END
	
#define REUSEADDR_SOCKET(_Socket) G_STMT_START{												\
	int       optval = 1;																									\
	setsockopt (Socket, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof (int));				\
	}G_STMT_END
			
#define BROADCAST_SOCKET(_Socket) G_STMT_START{												\
	int       optval = 1;																									\
	setsockopt(_Socket, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(int));				\
	}G_STMT_END	
	
#define KEEPALIVE_SOCKET(_Socket) G_STMT_START{												\
	int       optval = 1;																									\
	setsockopt(_Socket, SOL_SOCKET, SO_KEEPALIVE, &optval, sizeof(int));				\
	}G_STMT_END		
	
#define NONBLOCK_SOCKET(_Socket,_Etat) G_STMT_START{										\
	set_NonBlock (_Socket,_Etat);																					\
	}G_STMT_END	


#define SIGNAL_SOCKET(_Socket) G_STMT_START{										\
	signal_socket (_Socket);																				\
	}G_STMT_END
	
#define OPEN_SOCKET(_Socket,_sock,_Info_Demarrage,_domain,_type,_protocol) G_STMT_START{		\
		if ((_Socket = socket_control (_domain, _type, _protocol)) < 0)														\
			DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement)); 	\
		bzero (&_sock, sizeof (struct sockaddr));																					\
		strcpy (_sock.sa_data, _Info_Demarrage.Interface);																		\
		_sock.sa_family = _domain;																											\
		}G_STMT_END

		
#define SNAP_LEN 1514
#ifdef  __DARWIN_ENABLE
#define OPEN_SOCKET_PCAP(_captureHandle,_Info_Demarrage,_errbuf) 				G_STMT_START{																	\
	if ((_captureHandle=pcap_open_live(_Info_Demarrage.Interface,SNAP_LEN,IFF_PROMISC,1000,_errbuf)) == NULL) 	\
			DEBUG_ERROR("Unable to open the adapter. %s is not supported by WinPcap\nnpcap_open_live error : %s", _Info_Demarrage.nom,_errbuf);	 \
	}G_STMT_END			
#endif

#ifdef  __WINDOWS_ENABLE
#define OPEN_SOCKET_PCAP(_captureHandle,_Info_Demarrage,_errbuf) 				G_STMT_START{																					\
		if ( (_captureHandle = pcap_open(_Info_Demarrage.nom, 65535, PCAP_OPENFLAG_PROMISCUOUS, 1, NULL,  _errbuf ) ) == NULL) 	\
			DEBUG_ERROR("Unable to open the adapter. %s is not supported by WinPcap\nnpcap_open_live error : %s", Info_Demarrage.nom,_errbuf); 	\
		}G_STMT_END			
#endif

#define SEND(_Nb_Octet,_Socket,_Packet, _len) 				G_STMT_START{		\
		_Nb_Octet = sendto_control(_Socket,_Packet, _len, 0, NULL, 0);				\
		}G_STMT_END		
		
#define SENDTO(_Socket,_Packet, _len, _flags, _to, _tolen) 				G_STMT_START{					\
		if ( sendto_control(_Socket,_Packet, _len, _flags, (struct sockaddr *)_to, _tolen) == -1) 										\
			DEBUG_ERROR("Socket = %d   Erreur[%s]  size(%d)",_Socket, strerror (errno),_len);											\
		}G_STMT_END		


#define _SENDTO(_Socket,_Packet_Arp, _len, _flags, _to, _tolen) 				G_STMT_START{							\
		if ( sendto_control(_Socket,_Packet_Arp, _len, _flags, (struct sockaddr *)_to, _tolen) == -1) 										\
			DEBUG_ERROR("Socket = %d   Erreur[%s]  size(%d)",_Socket, strerror (errno),_len);													\
		}G_STMT_END		
	
		
#define SENDTO_PCAP(_captureHandle,_Packet_Arp, _len) 				G_STMT_START{																						\
		Nouveau_Octet_Envoye (_len);																																									\
		if (pcap_sendpacket(_captureHandle,(u_char *) _Packet_Arp, _len) != 0) 																								\
			DEBUG_ERROR( "Error sending the packet: \n", pcap_geterr(captureHandle));		 	\
		}G_STMT_END				
 			
#define BIND(_Socket,_sock) 				G_STMT_START{							\
		if (bind_control (_Socket,(struct sockaddr *)_sock) == -1) 				\
			DEBUG_ERROR("Bind : %s", strerror (errno));								\
		}G_STMT_END			
		
#define LISTEN(_Socket,_Nb_Max_Client)	G_STMT_START{						\
		if (listen_control (_Socket,_Nb_Max_Client)== -1) 								\
			DEBUG_ERROR("Listen : %s", strerror (errno));								\
		}G_STMT_END		

		
#ifndef         __WINDOWS_ENABLE
#define CLOSE_SOCKET(_Socket) 						G_STMT_START{			\
		if (close_control (_Socket) < 0)													\
			DEBUG_ERROR("%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));							\
		}G_STMT_END
#else
#define CLOSE_SOCKET(_Socket) 						G_STMT_START{			\
		if (close_control (_Socket) < 0)													\
			DEBUG_ERROR("Erreur(%d)  %s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)",WSAGetLastError(), strerror (errno), Get_Nb_Socket_Ouvert(), Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Sem_GList_Liste_Thread_Scan), g_list_length (GList_Equipement));							\
		}G_STMT_END					
#endif	

		
#define SHUTDOWN_SOCKET(_Socket) 						G_STMT_START{			\
		if (shutdown_socket (_Socket) < 0)													\
			DEBUG_WARNING("shutdown Socket (%d)",_Socket);							\
		}G_STMT_END
		
		
#define CLOSE_SOCKET_PCAP(_captureHandle) 	G_STMT_START{			\
		pcap_close(captureHandle);																\
		}G_STMT_END

#endif
